import { Construct } from 'constructs';
import * as cdktf from 'cdktf';
export interface MlModelConfig extends cdktf.TerraformMetaArguments {
    /**
    * The organization that developed the ML Model.
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#developed_by MlModel#developed_by}
    */
    readonly developedBy: string;
    /**
    * The execution configuration of the ML Model.
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#exec_config MlModel#exec_config}
    */
    readonly execConfig: MlModelExecConfig;
    /**
    * The family of the ML Model.
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#family MlModel#family}
    */
    readonly family: string;
    /**
    * The organization hosting the ML Model.
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#hosted_by MlModel#hosted_by}
    */
    readonly hostedBy: string;
    /**
    * A name that uniquely identifies this ML Model
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#name MlModel#name}
    */
    readonly name: string;
    /**
    * The visibility of the ML Model.
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#visibility MlModel#visibility}
    */
    readonly visibility: string;
}
export interface MlModelExecConfig {
    /**
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#base_url MlModel#base_url}
    */
    readonly baseUrl: string;
    /**
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#features MlModel#features}
    */
    readonly features: string[];
    /**
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#metadata MlModel#metadata}
    */
    readonly metadata: {
        [key: string]: string;
    };
    /**
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#model_identifier MlModel#model_identifier}
    */
    readonly modelIdentifier: string;
}
export declare function mlModelExecConfigToTerraform(struct?: MlModelExecConfig | cdktf.IResolvable): any;
export declare function mlModelExecConfigToHclTerraform(struct?: MlModelExecConfig | cdktf.IResolvable): any;
export declare class MlModelExecConfigOutputReference extends cdktf.ComplexObject {
    private isEmptyObject;
    private resolvableValue?;
    /**
    * @param terraformResource The parent resource
    * @param terraformAttribute The attribute on the parent resource this class is referencing
    */
    constructor(terraformResource: cdktf.IInterpolatingParent, terraformAttribute: string);
    get internalValue(): MlModelExecConfig | cdktf.IResolvable | undefined;
    set internalValue(value: MlModelExecConfig | cdktf.IResolvable | undefined);
    private _baseUrl?;
    get baseUrl(): string;
    set baseUrl(value: string);
    get baseUrlInput(): string | undefined;
    private _features?;
    get features(): string[];
    set features(value: string[]);
    get featuresInput(): string[] | undefined;
    private _metadata?;
    get metadata(): {
        [key: string]: string;
    };
    set metadata(value: {
        [key: string]: string;
    });
    get metadataInput(): {
        [key: string]: string;
    } | undefined;
    private _modelIdentifier?;
    get modelIdentifier(): string;
    set modelIdentifier(value: string);
    get modelIdentifierInput(): string | undefined;
}
/**
* Represents a {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model vellum_ml_model}
*/
export declare class MlModel extends cdktf.TerraformResource {
    static readonly tfResourceType = "vellum_ml_model";
    /**
    * Generates CDKTF code for importing a MlModel resource upon running "cdktf plan <stack-name>"
    * @param scope The scope in which to define this construct
    * @param importToId The construct id used in the generated config for the MlModel to import
    * @param importFromId The id of the existing MlModel that should be imported. Refer to the {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model#import import section} in the documentation of this resource for the id to use
    * @param provider? Optional instance of the provider where the MlModel to import is found
    */
    static generateConfigForImport(scope: Construct, importToId: string, importFromId: string, provider?: cdktf.TerraformProvider): cdktf.ImportableResource;
    /**
    * Create a new {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs/resources/ml_model vellum_ml_model} Resource
    *
    * @param scope The scope in which to define this construct
    * @param id The scoped construct ID. Must be unique amongst siblings in the same scope
    * @param options MlModelConfig
    */
    constructor(scope: Construct, id: string, config: MlModelConfig);
    private _developedBy?;
    get developedBy(): string;
    set developedBy(value: string);
    get developedByInput(): string | undefined;
    private _execConfig;
    get execConfig(): MlModelExecConfigOutputReference;
    putExecConfig(value: MlModelExecConfig): void;
    get execConfigInput(): MlModelExecConfig | cdktf.IResolvable | undefined;
    private _family?;
    get family(): string;
    set family(value: string);
    get familyInput(): string | undefined;
    private _hostedBy?;
    get hostedBy(): string;
    set hostedBy(value: string);
    get hostedByInput(): string | undefined;
    get id(): string;
    private _name?;
    get name(): string;
    set name(value: string);
    get nameInput(): string | undefined;
    private _visibility?;
    get visibility(): string;
    set visibility(value: string);
    get visibilityInput(): string | undefined;
    protected synthesizeAttributes(): {
        [name: string]: any;
    };
    protected synthesizeHclAttributes(): {
        [name: string]: any;
    };
}
