import { Construct } from 'constructs';
import * as cdktf from 'cdktf';
export interface VellumProviderConfig {
    /**
    * API Key to authenticate with the Vellum API
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs#api_key VellumProvider#api_key}
    */
    readonly apiKey?: string;
    /**
    * Base URL to use with the Vellum API
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs#base_url VellumProvider#base_url}
    */
    readonly baseUrl?: string;
    /**
    * Alias name
    *
    * Docs at Terraform Registry: {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs#alias VellumProvider#alias}
    */
    readonly alias?: string;
}
/**
* Represents a {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs vellum}
*/
export declare class VellumProvider extends cdktf.TerraformProvider {
    static readonly tfResourceType = "vellum";
    /**
    * Generates CDKTF code for importing a VellumProvider resource upon running "cdktf plan <stack-name>"
    * @param scope The scope in which to define this construct
    * @param importToId The construct id used in the generated config for the VellumProvider to import
    * @param importFromId The id of the existing VellumProvider that should be imported. Refer to the {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs#import import section} in the documentation of this resource for the id to use
    * @param provider? Optional instance of the provider where the VellumProvider to import is found
    */
    static generateConfigForImport(scope: Construct, importToId: string, importFromId: string, provider?: cdktf.TerraformProvider): cdktf.ImportableResource;
    /**
    * Create a new {@link https://registry.terraform.io/providers/vellum-ai/vellum/0.0.7/docs vellum} Resource
    *
    * @param scope The scope in which to define this construct
    * @param id The scoped construct ID. Must be unique amongst siblings in the same scope
    * @param options VellumProviderConfig = {}
    */
    constructor(scope: Construct, id: string, config?: VellumProviderConfig);
    private _apiKey?;
    get apiKey(): string | undefined;
    set apiKey(value: string | undefined);
    resetApiKey(): void;
    get apiKeyInput(): string | undefined;
    private _baseUrl?;
    get baseUrl(): string | undefined;
    set baseUrl(value: string | undefined);
    resetBaseUrl(): void;
    get baseUrlInput(): string | undefined;
    private _alias?;
    get alias(): string | undefined;
    set alias(value: string | undefined);
    resetAlias(): void;
    get aliasInput(): string | undefined;
    protected synthesizeAttributes(): {
        [name: string]: any;
    };
    protected synthesizeHclAttributes(): {
        [name: string]: any;
    };
}
